clear all
clc
global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c


%% ****************************  loading & specifying data ****************************


EstimOpt.fnameinput = ('DATA_boxcox.mat');
DATA = load(EstimOpt.fnameinput);

DATA.WTPh(DATA.WTPh == 1000000) = 1.5*DATA.WTPl(DATA.WTPh==1000000);
DATA.WTPh = DATA.WTPh./DATA.PPP;
DATA.WTPl = DATA.WTPl./DATA.PPP;
DATA.WTP = (DATA.WTPh+DATA.WTPl)/2;
DATA.WTP(DATA.WTPY == 0) = 0;
DATA.CTRdummy = dummyvar(DATA.CTR);

DATA.Xc = [DATA.male 2011-DATA.year DATA.EDUi==2 DATA.EDUi==3 DATA.EDUi==4 DATA.HH];

DATA.skip = DATA.WTP == 0 | isnan(DATA.INC) | isnan(sum(DATA.Xc,2));


%% ****************************  loading & specifying input ****************************


INPUT.Y = DATA.WTP;
INPUT.Y(DATA.skip) = [];
EstimOpt.NamesY = {'WTP'};


% INPUT.Xh = [DATA.INC];
% INPUT.Xh = [DATA.CTRdummy(:,1:8) DATA.INC];
% INPUT.Xh(DATA.WTP == 0 | isnan(DATA.INC),:) = [];
% EstimOpt.NamesH = {'Inc'};
% EstimOpt.NamesH = {'DK' 'EE' 'FI' 'DE' 'LV' 'LT' 'PL' 'RU' 'Inc'};
% INPUT.Xh = INPUT.Xh/(min(INPUT.Xh)/1.01);

%% **************************** estimation and optimization options ****************************


EstimOpt.EPS = 1.e-12; % overall precision level
EstimOpt.HessEstFix = 1;


%% **************************** Estimation ****************************


% **************************** base: -47897.1751 ****************************

INPUT.Xa = [DATA.CTRdummy DATA.INC];
INPUT.Xa(DATA.skip,:) = [];
EstimOpt.NamesA = {'DK' 'EE' 'FI' 'DE' 'LV' 'LT' 'PL' 'RU' 'SE', 'income'};

EstimOpt.b0 = [4.44160681945992;4.01645762141534;4.58224269096915;3.98495618983263;2.23235189183882;2.94124257449017;3.03784210490457;2.96102824986266;5.50407505938526;0.525899709082063;0.151473785886325;0.559024937129021;0.673547697077226];
EstimOpt.TransStruct = [1, NaN(1,9), 2]; % the same numbers for the same parameters, positive for box-cox transformation, negative for exponentiation, 0 for none

Results.BOXCOX_b = BOXCOX(INPUT,EstimOpt);
xlswrite('boxcox_b.xls', Results.BOXCOX_b.R_out)

% **************************** plot ****************************

sim_no = 1e6;
Xm = mean(INPUT.Xa(:,1:9));

B = Results.BOXCOX_b.bhat;
B_tmp = mvnrnd(Results.BOXCOX_b.bhat,Results.BOXCOX_b.ihess,sim_no-1)';
B = [B, B_tmp];
elasticity_all = zeros(50,sim_no);
INC = 0.1:0.1:5;
for i = 1:50    
    elasticity_all(i,1:sim_no) = B(10,:).*INC(i).^B(12,:) ./ (B(11,:).*(Xm*B(1:9,:) + B(10,:)/B(12,:).*(INC(i).^B(12,:)-1))+1);
end

elasticity = [elasticity_all(:,1)'; quantile(elasticity_all',0.025); quantile(elasticity_all',0.975)];

figure(1)
set(gcf,'defaultlinelinewidth',2);
set(gca,'Position',[.1 .1 .87 .82]);
set(gcf,'Color', 'w')
% set(gca,'YTick',[])
plot(INC,elasticity(1,:),INC,elasticity(2,:),'b--',INC,elasticity(3,:),'b--')
title('Figure 1. Income elasticity of WTP -- baseline model')
xlabel('Income [1000 EUR @ PPP]')
ylabel('Elasticity')
axis([0.5 5 0 1.2])
set(gca,'YTick', [0:0.1:1.2])

l1 = legend( 'Income elasticity of WTP', '95% confidence interval');
set(l1, 'Position', [.25,.80,.15,.01]);

% **************************** restricted model: -48598.3857 ****************************

EstimOpt.TransStruct = [1, NaN(1,9), 1]; 
EstimOpt.TransActive = zeros(11,1);
EstimOpt.NotActive = [0];
Results.BOXCOX_b_r = BOXCOX(INPUT,EstimOpt);

xlswrite('boxcox_b_r.xls', Results.BOXCOX_b_r.R_out)


TEST(1,1) = Results.BOXCOX_b.LL;
TEST(2,1) = Results.BOXCOX_b_r.LL;
[~, TEST(3,1), TEST(4,1), TEST(5,1)] = lratiotest(TEST(1,1),TEST(2,1),2);


% **************************** base + het: -47801.5856 ****************************

EstimOpt.b0 = [4.34937156665933;3.91805333484522;4.48587034741192;3.90852525406558;2.16742854305346;2.85213950593326;2.96751855592305;2.88433199372160;5.36801953587888;0.478608626157918;0.140705229056913;0.606095257478124;-0.0121641623384948;-0.0391982099660446;-0.114849015694549;-0.188277489283786;-0.182586872145397;-0.206920260458708;0.0521927290083415;0.0833016509114245;0.691249888774159];

INPUT.Xh = [DATA.CTRdummy(:,1:8)];
EstimOpt.NamesH = {'DK' 'EE' 'FI' 'DE' 'LV' 'LT' 'PL' 'RU'};
INPUT.Xh(DATA.skip,:) = [];

EstimOpt.TransStruct = [1, NaN(1,9), 2, -8:-1]; % the same numbers for the same parameters, positive for box-cox transformation, negative for exponentiation, 0 for none
EstimOpt.TransActive = [];

% EstimOpt.TransStruct = [1, NaN(1,9), 2, -ones(1,8)];
% EstimOpt.TransActive = [1, zeros(1,9), 1,zeros(1,8)];
% EstimOpt.NotActive = 1;
% EstimOpt.b0 = [EstimOpt.b0(1:end-9); EstimOpt.b0(end)];

Results.BOXCOX_bh = BOXCOX(INPUT,EstimOpt);
xlswrite('boxcox_bh.xls', Results.BOXCOX_bh.R_out)

% **************************** plot ****************************

sim_no = 1e6;
Xm = mean(INPUT.Xa(:,1:9));

B = Results.BOXCOX_bh.bhat;
B_tmp = mvnrnd(Results.BOXCOX_bh.bhat,Results.BOXCOX_bh.ihess,sim_no-1)';
B = [B, B_tmp];
elasticity_all = zeros(50,sim_no);
INC = 0.1:0.1:5;
for i = 1:50    
    elasticity_all(i,1:sim_no) = B(10,:).*INC(i).^B(12,:) ./ (B(11,:).*(Xm*B(1:9,:) + B(10,:)/B(12,:).*(INC(i).^B(12,:)-1))+1);
end

elasticity = [elasticity_all(:,1)'; quantile(elasticity_all',0.025); quantile(elasticity_all',0.975)];

figure(2)
set(gcf,'defaultlinelinewidth',2);
set(gca,'Position',[.1 .1 .87 .82]);
set(gcf,'Color', 'w')
% set(gca,'YTick',[])
plot(INC,elasticity(1,:),INC,elasticity(2,:),'b--',INC,elasticity(3,:),'b--')
title('Figure 2. Income elasticity of WTP -- model with heteroskedascity')
xlabel('Income [1000 EUR @ PPP]')
ylabel('Elasticity')
axis([0.5 5 0 1.2])
set(gca,'YTick', [0:0.1:1.2])

l1 = legend( 'Income elasticity of WTP', '95% confidence interval');
set(l1, 'Position', [.25,.80,.15,.01]);

% **************************** restricted model: -48119.3229 ****************************

EstimOpt.TransStruct = [1, NaN(1,9), 1, -8:-1]; 
EstimOpt.TransActive = [zeros(11,1); ones(8,1)];
EstimOpt.NotActive = [0];
Results.BOXCOX_bh_r = BOXCOX(INPUT,EstimOpt);

xlswrite('boxcox_bh_r.xls', Results.BOXCOX_bh_r.R_out)


TEST(1,2) = Results.BOXCOX_bh.LL;
TEST(2,2) = Results.BOXCOX_bh_r.LL;
[~, TEST(3,2), TEST(4,2), TEST(5,2)] = lratiotest(TEST(1,2),TEST(2,2),2);

% **************************** base + controls: -47851.3664 ****************************

INPUT.Xa = [DATA.CTRdummy DATA.INC DATA.Xc];
INPUT.Xa(DATA.skip,:) = [];
EstimOpt.NamesA = {'DK' 'EE' 'FI' 'DE' 'LV' 'LT' 'PL' 'RU' 'SE', 'income', 'male', 'age', 'edu2', 'edu3', 'edu4', 'hhsize'};   

INPUT.Xh = [];
EstimOpt.TransActive = [];

EstimOpt.TransStruct = [1, NaN(1,9), 2, NaN(1,6)]; % the same numbers for the same parameters, positive for box-cox transformation, negative for exponentiation, 0 for none

EstimOpt.b0 = [4.18248084662623;3.60768283033263;4.37045897928676;3.68885051905434;1.91975118937038;2.55999122123575;2.65055013332520;2.57716373808562;5.18418591198825;0.421946339520016;-0.200637407058809;0.000833533938118316;0.105024627225198;0.303419666790995;0.479210266008881;0.0335350946074845;0.151782867278487;0.536762781517020;0.670051593848904];

% EstimOpt.TransStruct = [1, NaN(1,9), 2, 3:8];
% EstimOpt.b0 = [EstimOpt.b0(1:end-1); ones(6,1); EstimOpt.b0(end)];

Results.BOXCOX_bc = BOXCOX(INPUT,EstimOpt);
xlswrite('boxcox_bc.xls', Results.BOXCOX_bc.R_out)

% **************************** plot ****************************

sim_no = 1e6;
Xm = mean(INPUT.Xa(:,1:9));
Xc = mean(INPUT.Xa(:,11:end));

B = Results.BOXCOX_bc.bhat;
B_tmp = mvnrnd(Results.BOXCOX_bc.bhat,Results.BOXCOX_bc.ihess,sim_no-1)';
B = [B, B_tmp];
elasticity_all = zeros(50,sim_no);
INC = 0.1:0.1:5;
for i = 1:50    
    elasticity_all(i,1:sim_no) = B(10,:).*INC(i).^B(18,:) ./ (B(17,:).*(Xm*B(1:9,:) + Xc*B(11:16,:) + B(10,:)/B(18,:).*(INC(i).^B(18,:)-1))+1);
end

elasticity = [elasticity_all(:,1)'; quantile(elasticity_all',0.025); quantile(elasticity_all',0.975)];

figure(3)
set(gcf,'defaultlinelinewidth',2);
set(gca,'Position',[.1 .1 .87 .82]);
set(gcf,'Color', 'w')
% set(gca,'YTick',[])
plot(INC,elasticity(1,:),INC,elasticity(2,:),'b--',INC,elasticity(3,:),'b--')
title('Figure 3. Income elasticity of WTP -- model with controls')
xlabel('Income [1000 EUR @ PPP]')
ylabel('Elasticity')
axis([0.5 5 0 1.2])
set(gca,'YTick', [0:0.1:1.2])

l1 = legend( 'Income elasticity of WTP', '95% confidence interval');
set(l1, 'Position', [.25,.80,.15,.01]);

% **************************** restricted model: -48553.5376 ****************************

EstimOpt.TransStruct = [1, NaN(1,9), 1, NaN(1,6)]; 
EstimOpt.TransActive = zeros(17,1);
EstimOpt.NotActive = [0];
Results.BOXCOX_bc_r = BOXCOX(INPUT,EstimOpt);

xlswrite('boxcox_bc_r.xls', Results.BOXCOX_bc_r.R_out)


TEST(1,3) = Results.BOXCOX_bc.LL;
TEST(2,3) = Results.BOXCOX_bc_r.LL;
[~, TEST(3,3), TEST(4,3), TEST(5,3)] = lratiotest(TEST(1,3),TEST(2,3),2);


% **************************** base + controls + het: -47761.3722 ****************************

% EstimOpt.b0 = [EstimOpt.b0(1:end-9); ones(8,1); EstimOpt.b0(end)];
EstimOpt.b0 = [4.13114892653017;3.56300955069544;4.30877582257944;3.65370501145229;1.90078045450441;2.52534871566948;2.62988788575738;2.55360041246823;5.09841487116198;0.391195861392956;-0.176428751654644;0.000259756176493755;0.103896062873992;0.275860477646429;0.428385155391954;0.0319933185289773;0.140995330130835;0.588381347897576;0.000631889908348633;-0.0320608437797117;-0.107569090183273;-0.178798955725876;-0.177847910456133;-0.189918384794662;0.0566537546491548;0.0842718766717407;0.681801235814144];

INPUT.Xh = [DATA.CTRdummy(:,1:8)];
INPUT.Xh(DATA.skip,:) = [];

EstimOpt.TransStruct = [1, NaN(1,9), 2, NaN(1,6),-8:-1]; % the same numbers for the same parameters, positive for box-cox transformation, negative for exponentiation, 0 for none
EstimOpt.TransActive = [];


Results.BOXCOX_bch = BOXCOX(INPUT,EstimOpt);
xlswrite('boxcox_bch.xls', Results.BOXCOX_bch.R_out)

% **************************** plot ****************************

sim_no = 1e6;
Xm = mean(INPUT.Xa(:,1:9));
Xc = mean(INPUT.Xa(:,11:end));

B = Results.BOXCOX_bch.bhat;
B_tmp = mvnrnd(Results.BOXCOX_bch.bhat,Results.BOXCOX_bch.ihess,sim_no-1)';
B = [B, B_tmp];
elasticity_all = zeros(50,sim_no);
INC = 0.1:0.1:5;
for i = 1:50    
    elasticity_all(i,1:sim_no) = B(10,:).*INC(i).^B(18,:) ./ (B(17,:).*(Xm*B(1:9,:) + Xc*B(11:16,:) + B(10,:)/B(18,:).*(INC(i).^B(18,:)-1))+1);
end

elasticity = [elasticity_all(:,1)'; quantile(elasticity_all',0.025); quantile(elasticity_all',0.975)];

figure(4)
set(gcf,'defaultlinelinewidth',2);
set(gca,'Position',[.1 .1 .87 .82]);
set(gcf,'Color', 'w')
% set(gca,'YTick',[])
plot(INC,elasticity(1,:),INC,elasticity(2,:),'b--',INC,elasticity(3,:),'b--')
title('Figure 4. Income elasticity of WTP -- model with controls and heteroskedascity')
xlabel('Income [1000 EUR @ PPP]')
ylabel('Elasticity')
axis([0.5 5 0 1.2])
set(gca,'YTick', [0:0.1:1.2])
l1 = legend( 'Income elasticity of WTP', '95% confidence interval');
set(l1, 'Position', [.25,.80,.15,.01]);


% **************************** restricted model: -48077.1279 ****************************

EstimOpt.TransStruct = [1, NaN(1,9), 1, NaN(1,6), -8:-1 ]; 
EstimOpt.TransActive = [zeros(17,1); ones(8,1)];
EstimOpt.NotActive = [0];
Results.BOXCOX_bch_r = BOXCOX(INPUT,EstimOpt);

xlswrite('boxcox_bch_r.xls', Results.BOXCOX_bch_r.R_out)

TEST(1,4) = Results.BOXCOX_bch.LL;
TEST(2,4) = Results.BOXCOX_bch_r.LL;
[~, TEST(3,4), TEST(4,4), TEST(5,4)] = lratiotest(TEST(1,4),TEST(2,4),2);

